package cn.chnmuseum.party.web.controller;

import cn.chnmuseum.party.auth.SHA256PasswordEncryptionService;
import cn.chnmuseum.party.auth.util.AESUtils;
import cn.chnmuseum.party.auth.util.JwtTokenUtil;
import cn.chnmuseum.party.common.checkcode.SpecCaptcha;
import cn.chnmuseum.party.common.enums.AuditOperationEnum;
import cn.chnmuseum.party.model.Menu;
import cn.chnmuseum.party.model.Role;
import cn.chnmuseum.party.model.TUser;
import cn.chnmuseum.party.service.RoleService;
import cn.chnmuseum.party.service.impl.MenuServiceImpl;
import cn.chnmuseum.party.service.impl.TUserServiceImpl;
import cn.chnmuseum.party.web.controller.base.BaseController;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.hutool.extra.qrcode.QrConfig;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * shiro权限控制登录Controller
 */
@Api(tags = "登录接口")
@RestController
public class LoginController extends BaseController {

    private static final Logger LOGGER = LoggerFactory.getLogger(LoginController.class);

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    private TUserServiceImpl userService;

    @Resource
    private RoleService roleService;

    @Resource
    private MenuServiceImpl menuService;

    @Resource
    private SysLogController sysLogController;

    private static final String SHIRO_JWT_TOKEN = "shiro:jwt:token:";
    //用户登录次数计数  redisKey 前缀
    private String SHIRO_LOGIN_COUNT = "shiro_login_count_";
    //用户登录是否被锁定    一小时 redisKey 前缀
    private String SHIRO_IS_LOCK = "shiro_is_lock_";

    // 能够登录的角色,平台管理员和统计管理员
    private String ROLE_XTGLY = "xtgly";
    private String ROLE_TJGLY = "tjgly";

    // 未授权跳转的页面
    @RequestMapping(value = "403", method = RequestMethod.GET)
    public void noPermissions(HttpServletResponse response) throws IOException {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json;charset=utf-8");
        response.setStatus(403);
        response.getWriter().write("{\"status\":403,\"message\":\"Unauthorized!\"}");
    }

    /**
     * 管理员ajax登录请求 后端用户登录
     *
     * @param username
     * @param password
     * @return
     */
    @RequestMapping(value = "ajaxLogin", method = RequestMethod.POST)
    public Map<String, Object> ajaxLogin(@RequestParam(value = "username", required = true) String username,
                                         @RequestParam(value = "password", required = true) String password,
//                                                         @RequestParam(value = "captcha", required = true) String captcha,
                                         HttpServletRequest request) {
        Map<String, Object> resultMap = new LinkedHashMap<>();
//        String captchaId = request.getHeader("CaptchaId");
//        if (StringUtils.isNotBlank(captcha)) {
//            if (StringUtils.isNotBlank(captchaId)) {
//                String uuidCap = stringRedisTemplate.opsForValue().get(captchaId);
//                if (StringUtils.isNotBlank(uuidCap)) {
//                    if (!uuidCap.trim().equalsIgnoreCase(captcha.trim())) {
//                        stringRedisTemplate.delete(captchaId);
//                        resultMap.put("status", 400);
//                        resultMap.put("message", "验证码不正确！");
//                        return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
//                    }
//                } else {
//                    stringRedisTemplate.delete(captchaId);
//                    resultMap.put("status", 400);
//                    resultMap.put("message", "验证码已失效，请刷新页面！");
//                    return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
//                }
//            } else {
//                //stringRedisTemplate.delete(captchaId);
//                resultMap.put("status", 400);
//                resultMap.put("message", "验证码获取失败！");
//                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
//            }
//        } else {
//            stringRedisTemplate.delete(captchaId);
//            resultMap.put("status", 400);
//            resultMap.put("message", "验证码不能为空！");
//            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
//        }
//        stringRedisTemplate.delete(captchaId);

        TUser user;
        if (StringUtils.isNoneBlank(username)) {

            try {
                //访问一次，计数一次
//                ValueOperations<String, String> opsForValue = stringRedisTemplate.opsForValue();
//                if ("LOCK".equals(opsForValue.get(SHIRO_IS_LOCK + username))) {
//                    resultMap.put("resultCode", "500");
//                    resultMap.put("message", "由于密码输入错误次数大于5次，12小时内帐号已禁止登录！请您联系相关管理人员。");
//                    return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
//                }

                user = userService.selectByUsername(username);
                if (user == null) {
                    resultMap.put("resultCode", "400");
                    resultMap.put("message", "用户名不正确！");
                    return resultMap;
                }

                if (AuditOperationEnum.DISABLE.name().equals(user.getStatus())) {
                    resultMap.put("resultCode", "400");
                    resultMap.put("message", "此帐号已禁用，请联系管理员！");
                    return resultMap;
                }

                if (user.getPermanent() != null && !user.getPermanent()) {
                    if (user.getEffectiveDate() != null && user.getEffectiveDate().isAfter(LocalDate.now()) || user.getExiredDate() != null && user.getExiredDate().isBefore(LocalDate.now())) {
                        resultMap.put("resultCode", "4F00");
                        resultMap.put("message", "此帐号已失效，请联系管理员！");
                        return resultMap;
                    }
                }
                //密码解密
                password = AESUtils.aesDecrypt(password);

                byte[] salt = user.getPasswordSalt();
                String s1 = new String(SHA256PasswordEncryptionService.createPasswordHash(password, salt));
                if (!s1.equals(new String(user.getPasswordHash()))) {
//                    opsForValue.increment(SHIRO_LOGIN_COUNT + username, 1);
//                    //计数大于5时，设置用户被锁定12小时
//
//                    //测试设置5000次
//                    int i = 5000;
//                    String s = opsForValue.get(SHIRO_LOGIN_COUNT + username);
//                    if (StringUtils.isNotBlank(s)) {
//                        if (Integer.parseInt(s) >= i) {
//                            opsForValue.set(SHIRO_IS_LOCK + username, "LOCK");
//                            stringRedisTemplate.expire(SHIRO_IS_LOCK + username, 12, TimeUnit.HOURS);
//                        }
//                    }
                    resultMap.put("resultCode", "400");
                    resultMap.put("message", "密码不正确！");
//                    resultMap.put("message", "密码不正确,您还有" + (i - Integer.parseInt(s)) + "次机会！");
                    return resultMap;
                }
                List<Role> roles = roleService.selectRoleByUserId(user.getId());

                // 该参数判断登录请求：1是大屏，2是h5,改参数为空是PC
                String administrator = request.getParameter("administrator");
                List<String> roleAliasList = roles.stream().map(Role::getAlias).collect(Collectors.toList());
                if (StringUtils.isNotBlank(administrator) && "1".equals(administrator)) {
                    if (!roleAliasList.contains(ROLE_TJGLY) && !roleAliasList.contains(ROLE_XTGLY)) {
                        resultMap.put("resultCode", "400");
                        resultMap.put("message", "您登录的账号既不是平台用户账号也不是统计用户账号，不能查看大屏");
                        return resultMap;
                    }
                }
                if (StringUtils.isNotBlank(administrator) && "2".equals(administrator)) {
                    if (!"2".equals(user.getType()) && !"4".equals(user.getType())) {
                        resultMap.put("resultCode", "400");
                        resultMap.put("message", "您登录的账号既不是运维账号也不是单位用户账号，不能使用h5的功能");
                        return resultMap;
                    }
                }
                // PC
                if (StringUtils.isEmpty(administrator)) {
                    if ("3".equals(user.getType()) || "4".equals(user.getType())) {
                        resultMap.put("resultCode", "400");
                        resultMap.put("message", "您登录的账号是机顶盒账号或运维账号，不能使用PC的功能");
                        return resultMap;
                    }
                }

                List<String> list1 = new ArrayList<>();
                //获取当前用户角色拥有菜单
                List<Menu> userMenuPerms = new ArrayList<>();
                if (roles != null && roles.size() > 0 && roles.get(0) != null) {
                    roles.forEach(r -> list1.add(r.getId()));
                    user.setRoleList(list1);
                    userMenuPerms = this.menuService.getUserMenuPerms(list1);
                }

                //登录时插入系统日志
                String operationContent = username + "登录本系统";
                if (user.getOrgName() != null) {
                    operationContent += "，机构" + user.getOrgName();
                }
                this.sysLogController.insertSysLog(operationContent, user);

                String token = JwtTokenUtil.sign(username, user.getId());
                // 将token信息存入Redis
                stringRedisTemplate.opsForValue().set(SHIRO_JWT_TOKEN + token, user.getId(), 12, TimeUnit.HOURS);
                String firstPassword = new String(SHA256PasswordEncryptionService.createPasswordHash("gb123456", salt));
                if (firstPassword.equals(new String(user.getPasswordHash()))) {
                    resultMap.put("isDefault", true);
                }
                if (user.getExiredDate() != null) {
                    Long i = user.getExiredDate().toEpochDay() - LocalDate.now().toEpochDay();
                    resultMap.put("expireNum", i);
                }
                user.setPassword("");
                user.setPhone("");
                resultMap.put("user", user);
                resultMap.put("token", token);
                resultMap.put("menuList", userMenuPerms);
                resultMap.put("resultCode", "200");
                resultMap.put("message", "登录成功");
                return resultMap;
            } catch (Exception e) {
                resultMap.put("resultCode", "500");
                resultMap.put("message", "登录失败");
            }
        }
        return getFailResult("登录失败");
    }

    @RequestMapping(value = "logout", method = RequestMethod.GET)
    public Map<String, Object> logout() {
        String token = request.getHeader("Authorization");
        if (token.startsWith("Bearer ")) {
            token = token.substring(7);
        }
        try {
            if (StringUtils.isNotBlank(token)) {
//                SecurityUtils.getSubject().logout();
                this.stringRedisTemplate.delete(SHIRO_JWT_TOKEN + token);
            }
            JSONObject resultMap = new JSONObject();
            resultMap.put("resultCode", "200");
            resultMap.put("message", "成功");
            resultMap.put("data", "");
            return resultMap;
        } catch (Exception e) {
            LOGGER.error("注销错误", e);
        }
        return getFailResult("注销错误");
    }


    @RequestMapping(value = "logoutUser", method = RequestMethod.GET)
    public Map<String, Object> logoutUser(String id) {
        try {
            Set<String> keys = stringRedisTemplate.keys(SHIRO_JWT_TOKEN + "*");
            ValueOperations<String, String> value = stringRedisTemplate.opsForValue();
            for (String key : keys) {
                if (id.equals(value.get(key))){
                    stringRedisTemplate.delete(key);
                }
            }
            JSONObject resultMap = new JSONObject();
            resultMap.put("resultCode", "200");
            resultMap.put("message", "成功");
            resultMap.put("data", "");
            return resultMap;
        } catch (Exception e) {
            LOGGER.error("注销错误", e);
        }
        return getFailResult("注销错误");
    }

    @RequestMapping(value = {"/verifyCode"}, method = {RequestMethod.GET})
    public void verifyCode(HttpServletResponse response, Integer width, Integer height) {
        String uuid = UUID.randomUUID().toString();
        try {
            SpecCaptcha iVerifyCodeGen;
            if ((width != null && width > 0) && (height != null && height > 0)) {
                iVerifyCodeGen = new SpecCaptcha(width, height);
            } else {
                iVerifyCodeGen = new SpecCaptcha();
            }
            response.addHeader("CaptchaId", uuid);
            response.setHeader("Pragma", "no-cache");
            response.setHeader("Cache-Control", "no-cache");
            response.setDateHeader("Expires", 0L);
            response.setContentType("image/jpeg");
            String out = iVerifyCodeGen.out(response.getOutputStream());
            this.stringRedisTemplate.opsForValue().set(uuid, out, 1800L);
        } catch (IOException e) {
            LOGGER.info("", e);
        }
    }

    @ApiOperation(value = "H5/PAD登录二维码", notes = "H5/PAD登录二维码", httpMethod = "GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "clientType", value = "客户端类型", dataType = "String", paramType = "query", allowableValues = "PAD, H5"),
            @ApiImplicitParam(name = "width", value = "二维码宽度", dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "height", value = "二维码高度", dataType = "int", paramType = "query")
    })
    @GetMapping(value = "loginByQrCode")
    public ResponseEntity<byte[]> loginByQrCode(
            @RequestParam(value = "clientType") String clientType,
            @RequestParam(value = "width", required = false, defaultValue = "150") int width,
            @RequestParam(value = "height", required = false, defaultValue = "150") int height,
            HttpServletRequest request) {
        String regUrl;
        if ("H5".equalsIgnoreCase(clientType)) {
//            regUrl = request.getScheme() + "://" + request.getServerName() + ":8088/h5/login";
            regUrl = request.getScheme() + "://" + request.getServerName() + "/h5/login";
        } else {
//            regUrl = request.getScheme() + "://" + request.getServerName() + ":8088/plat/#/login";
            regUrl = request.getScheme() + "://" + request.getServerName() + "/wdzcfwpt/";
        }
        QrConfig config = new QrConfig(width, height);
        config.setCharset(StandardCharsets.UTF_8);
        config.setMargin(1);
        config.setWidth(width);
        config.setHeight(height);
        final byte[] bytes = QrCodeUtil.generatePng(regUrl, config);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.IMAGE_PNG);
        headers.setContentLength(bytes.length);
        return new ResponseEntity<>(bytes, headers, HttpStatus.OK);
    }

    @RequestMapping(value = {"/verifyCode1"}, method = {RequestMethod.GET}, produces = MediaType.IMAGE_JPEG_VALUE)
    public @ResponseBody
    byte[] verifyCode1(HttpServletResponse response, Integer width, Integer height) {
        String uuid = UUID.randomUUID().toString();
        ByteArrayOutputStream baos = null;
        try {
            SpecCaptcha iVerifyCodeGen;
            if ((width != null && width > 0) && (height != null && height > 0)) {
                iVerifyCodeGen = new SpecCaptcha(width, height);
            } else {
                iVerifyCodeGen = new SpecCaptcha();
            }
            response.setHeader("Pragma", "no-cache");
            response.setHeader("Cache-Control", "no-cache");
            response.setDateHeader("Expires", 0L);
            response.setHeader("Access-Control-Allow-Headers", "CaptchaId");
            response.setHeader("Access-Control-Expose-Headers", "CaptchaId");
            response.addHeader("CaptchaId", uuid);

            baos = new ByteArrayOutputStream();
            String out = iVerifyCodeGen.out(baos);
            this.stringRedisTemplate.opsForValue().set(uuid, out, 1800L);
            return baos.toByteArray();
        } catch (Exception e) {
            LOGGER.info("", e);
        } finally {
            if (baos != null) {
                try {
                    baos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

}
